<?php

namespace Drupal\custom_library\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure Custom Library settings for this site.
 *
 * @internal
 */
class CSSAdvanceSettings extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'custom_library_css_advance_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['custom_library.assets.css_advance_settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $config = $this->config('custom_library.assets.css_advance_settings');
    // Get existing rows or set initial.
    $rows = $form_state->get('input_rows');
    if ($rows === NULL) {
      $rows = $config->get('input_rows') ?? [];
      if (empty($rows)) {
        $rows[] = ['url' => '', 'weight' => ''];
      }
      $form_state->set('input_rows', $rows);
    }
    $form['enable_custom_asset'] = [
      '#type' => 'checkbox',
      '#title' => 'Enable Advance Custom Library for CSS',
      '#default_value' => $config->get('enable_custom_asset'),
    ];
    $form['operations'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'config-setting-custom-library'],
    ];
    $form['operations']['css_settings_tabs'] = [
      '#type' => 'vertical_tabs',
    ];
    $form['operations']['css_operations'] = [
      '#type' => 'details',
      '#title' => $this->t('CSS Asset Settings'),
      '#attributes' => ['id' => 'js-advance-settings'],
      '#open' => TRUE,
      '#group' => 'css_settings_tabs',
    ];

    // Wrapper for AJAX.
    $form['operations']['css_operations']['#prefix'] = '<div id="table-wrapper">';
    $form['operations']['css_operations']['#suffix'] = '</div>';

    $form['operations']['css_operations']['my_table'] = [
      '#type' => 'table',
      '#header' => ['Asset Link', 'Weight'],
      '#empty' => $this->t('No rows found.'),
    ];
    foreach ($rows as $index => $row) {
      $form['operations']['css_operations']['my_table'][$index]['url'] = [
        '#type' => 'textfield',
        '#default_value' => $row['url'],
      ];
      $form['operations']['css_operations']['my_table'][$index]['weight'] = [
        '#type' => 'number',
        '#default_value' => $row['weight'],
      ];
    }

    // Add row button.
    $form['operations']['css_operations']['add_row'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add row'),
      '#submit' => ['::addRowSubmit'],
      '#ajax' => [
        'callback' => '::ajaxCallback',
        'wrapper' => 'table-wrapper',
      ],
      '#limit_validation_errors' => [],
    ];
    $form['operations']['css_operations']['remove'] = [
      '#type' => 'submit',
      '#value' => $this->t('Remove'),
      '#submit' => ['::removeRowSubmit'],
      '#ajax' => [
        'callback' => '::ajaxCallback',
        'wrapper' => 'table-wrapper',
      ],
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $submitted_rows = [];
    $table_input = $form_state->getValue('my_table') ?? [];

    foreach ($table_input as $row) {
      $submitted_rows[] = [
        'url' => $row['url'],
        'weight' => $row['weight'],
      ];
    }

    $this->config('custom_library.assets.css_advance_settings')
      ->set('enable_custom_asset', $form_state->getValue('enable_custom_asset'))
      ->set('input_rows', $submitted_rows)
      ->save();

    $form_state->set('input_rows', $submitted_rows);

    parent::submitForm($form, $form_state);
    drupal_flush_all_caches();
  }

  /**
   * Add a new empty row.
   */
  public function addRowSubmit(array &$form, FormStateInterface $form_state) {
    $rows = $form_state->get('input_rows');
    $rows[] = ['url' => '', 'weight' => ''];
    $form_state->set('input_rows', $rows);
    $form_state->setRebuild();
  }

  /**
   * AJAX callback.
   */
  public function ajaxCallback(array &$form, FormStateInterface $form_state) {
    return $form['operations'];
  }

  /**
   * Remove a specific row.
   */
  public function removeRowSubmit(array &$form, FormStateInterface $form_state) {
    // Get current table input values.
    $table_values = $form_state->getValue('my_table') ?? [];

    // Remove the row.
    array_pop($table_values);

    // Re-index the array to avoid gaps.
    $table_values = array_values($table_values);

    // Save to form state (so rebuild picks it up).
    $form_state->set('input_rows', $table_values);

    // Rebuild the form.
    $form_state->setRebuild();
  }

}
